import RemoteApi_pb2
import PIDataReferenceFrame_pb2
import remoteapi.PluginApi
import remoteapi

"""
ScenarioHelper.py

Helper methods for creating scenarios and entities.
"""

def newScenario(piData, refYear=2015, classification='UNCLASSIFIED', classColor=0x00ff0080, origin=[0,0,0], scenDesc='Python Test'):
	"""
	Creates a new file-based scenario.
	"""
	piData.deleteScenario()
	piData.initializeScenario(scenDesc, 0, 0, 0)
	piData.setOriginLLA(origin[0], origin[1], origin[2])
	piData.setReferenceYear(refYear)
	piData.setClassification(classification, classColor)

def newAddPlatformCommand(client, localId, name='Python Platform', icon='x'):
	"""
	Returns a create-platform command using the supplied localId on the given client.
	Note that you will need to call points.add() to create points on the platform.
	"""
	# Create a platform with a data point at time 0
	command = RemoteApi_pb2.Command()
	create = command.piData.platform.create
	create.header.name = name
	create.header.originalId = localId
	create.header.iconName = icon
	create.suppliedId = localId
	create.frame.system = PIDataReferenceFrame_pb2.COORDSYS_LLA
	return command

def createPlatform(client, localId, lla=[0,0,0], name="Python Platform", firstPointTime=0.0, lastPointTime=10.0, icon='x', ori=None):
	"""
	Creates a simple platform with two points at the given lat/lon (radians).
	"""
	command = newAddPlatformCommand(client, localId, name, icon)
	create = command.piData.platform.create

	# Create 2 points at 0 and 10
	p1 = create.points.add()
	p1.time = firstPointTime
	p1.position.x = lla[0]
	p1.position.y = lla[1]
	p1.position.z = lla[2]
	if ori is not None:
		p1.orientation.psi_yaw = ori[0]
		p1.orientation.theta_pitch = ori[1]
		p1.orientation.phi_roll = ori[2]
	p2 = create.points.add()
	p2.time = lastPointTime
	p2.position.x = lla[0]
	p2.position.y = lla[1]
	p2.position.z = lla[2]
	if ori is not None:
		p2.orientation.psi_yaw = ori[0]
		p2.orientation.theta_pitch = ori[1]
		p2.orientation.phi_roll = ori[2]
	return remoteapi.PluginApi.ResponseHandler(client, client.send(command),
			lambda r: r)

def createPlatformVelocity(client, localId, lla=[0,0,0], vel=[0,0,0], name="Python Platform", firstPointTime=0.0, lastPointTime=10.0):
	"""
	Creates a simple platform with two points at the given lat/lon (radians).
	"""
	command = newAddPlatformCommand(client, localId, name)
	create = command.piData.platform.create

	# Create 2 points at 0 and 10
	p1 = create.points.add()
	p1.time = firstPointTime
	p1.position.x = lla[0]
	p1.position.y = lla[1]
	p1.position.z = lla[2]
	p1.velocity.vx = vel[0]
	p1.velocity.vy = vel[1]
	p1.velocity.vz = vel[2]
	p2 = create.points.add()
	p2.time = lastPointTime
	p2.position.x = lla[0]
	p2.position.y = lla[1]
	p2.position.z = lla[2]
	p2.velocity.vx = vel[0]
	p2.velocity.vy = vel[1]
	p2.velocity.vz = vel[2]
	return remoteapi.PluginApi.ResponseHandler(client, client.send(command),
			lambda r: r)
