﻿from remoteapi import *


"""
TestCustomRendering.py

Unit test for the Remote API using Python that validates the creating
and updating custom rendering

"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()

pi.config().setResponsesEnabled(True)

f = piData.isScenarioInitialized()
if not f.isOk(1000):
	raise Exception("Failed isScenarioInitialized")

if f.value():
	f = piData.deleteScenario()
	if not f.isOk(5000):
		raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(5000):
	raise Exception("Failed to initialize scenario")

f = piData.setOriginLLA(0.1, 0.2, 1000)
if not f.isOk(1000):
	raise Exception("Failed to set OriginLLA")
	
f = piData.setReferenceYear(2013)
if not f.isOk(1000):
	raise Exception("Failed to set ref year")
	
# Test Platforms
f = piData.createPlatform("Platform 1", "x", 47)
if not f.isOk(10000):
	raise Exception("Failed to create platform")
platformId = f.value();

# Test Custom Rendering
f = piData.createCustomRendering(platformId, "Custom Rendering 1", "Custom Line", 86)
if not f.isOk(10000):
	raise Exception("Failed to create custom rendering")
customId = f.value();

f = piData.getCustomRenderingHeader(customId)
if not f.isOk(10000):
	raise Exception("Failed to get custom rendering")
if f.value().originalId != 86:
    raise Exception("Failed to get correct custom rendering")
	
# Test Custom Rendering
f = piData.updateCustomRenderingHeader(customId, "Custom Rendering 1", "Custom Line", 99)
if not f.isOk(10000):
	raise Exception("Failed to update custom rendering header")

f = piData.getCustomRenderingHeader(customId)
if not f.isOk(10000):
	raise Exception("Failed to get custom rendering")
if f.value().originalId != 99:
    raise Exception("Failed to get correct custom rendering")
	
client.close()
print("Test successful")