﻿import io
from remoteapi import *
import PISIMDISDraw_pb2


"""
TestGetAllGogObjects.py

Unit test for the Remote API using Python that validates the sending
of all provisional GOG objects.

"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
    raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

pi.config().setResponsesEnabled(True)

# If a scenario already exists, then we'll get initialization errors in SIMDIS if we try to initialize
f = piData.isScenarioInitialized()
if not f.isOk(1000):
    raise Exception("Failed isScenarioInitialized")

if f.value():
    f = piData.deleteScenario()
    if not f.isOk(5000):
        raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(5000):
    raise Exception("Failed to initialize scenario")

# Load a provisional GOG into SIMDIS
gogString = """
version 2
start
line
linewidth 2
3d name "X axis"
linecolor red
3d follow cpr
xy 0 0 0
xy 0 100 0
end
start
line
linewidth 2
3d name "Y axis"
linecolor green
3d follow cpr
xy 0 0 0
xy -100 0 0
end
start
line
linewidth 2
3d name "Z axis"
linecolor blue
3d follow cpr
xy 0 0 0
xy 0 0 100
end
"""

f = piSimdis.loadGogStream(gogString, 0, 1) # load a GOG with id=1
if not f.isOk(10000):
    raise Exception("Failed to load GOG stream")

# Get all the provisional GOG objects in SIMDIS
f = piSimdis.getAllGogObjects(PISIMDISDraw_pb2.GOG_TYPE_FINALIZED_AND_PROVISIONAL)
if not f.isOk(10000):
    raise Exception("Failed to get all GOG objects")

# See that the GOG we loaded is in there
gogFound = False
for gogId in f.value():
    if gogId == 1: # if there is a GOG with id=1, we found it
        gogFound = True
if not gogFound:
    raise Exception("Failed to get the GOG object that was loaded")

# Serialize the GOG that we loaded earlier
f = piSimdis.serializeGogToStream(1) # serialize the GOG with id=1
print(f.value().decode('utf-8')) # view the data in the console to confirm that it is correct
if not f.isOk(10000):
    raise Exception("Failed to serialize the GOG that was loaded")

client.close()
print("Test successful")
