import datetime
import time
import PICommon_pb2
import PIDataScenario_pb2
import remoteapi

"""
TestLiveScenario.py

Unit test for Remote API that test commands that require a live scenario
"""
client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()

pi.config().setResponsesEnabled(True)

# delete the scenario
f = piData.deleteScenario()
if not f.isOk(5000):
	raise Exception("Failed to delete scenario")

# create a new live scenario to test live-only commands
f = piData.initializeScenario("Unit test live mode", 0, 0, True)
if not f.isOk(1000):
	raise Exception("Failed to initialize live scenario")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piData.setReferenceYear(currDateTime.year)
if not f.isOk(1000):
	raise Exception("Failed to set reference year")
piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_FREEWHEEL, secsSinceYear)
piData.clockSetTime(secsSinceYear)

# Create a platform to add live data points to
f = piData.createPlatform("Unit test live platform", "x", 1)
if not f.isOk(1000):
	raise Exception("Failed to create a new platform")
platId = f.value()

# Test start recording
f = piCommon.startRecording("$(TMP)/OkToDeleteThisFile.asi", True, False, 1)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Add data to the platform
for i in range(10):
	currDateTime = datetime.datetime.utcnow()
	startOfYear = datetime.datetime(currDateTime.year,1,1)
	secsSinceYear = (currDateTime - startOfYear).total_seconds()
	f = piData.addPlatformData(platId, secsSinceYear, 0, 0, 0)

	time.sleep(1)

# Test stop recording
f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_STEP, piData.clockGetBegin().value())
piData.clockStop()