import math
import re
import time
import Constants_pb2
import RemoteApi_pb2
import PIDataReferenceFrame_pb2
import PISIMDISFile_pb2
import remoteapi
import ScenarioHelper

"""
TestRangeConfiguration.py

Unit test for Remote API that creates a simple scenario and calls some range configuration methods.
"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
pi.config().setResponsesEnabled(True)
piData = pi.data()

ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
ScenarioHelper.createPlatform(client, 10, [math.radians(22), math.radians(-159), 0.0], 'P1')
ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 0.0], 'P2') # slightly north

# Fix time and wait for the response
piData.clockAutoConfigureBeginEnd().waitForResponse(10000)

# Now calculate some values
piSimdis = pi.simdis()

configuration = """<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>
  <RangeToolConfigurationFile>
  <Version>1.0</Version>
  <EntityPair>
  <StartEntity Type=\"P\" OriginalId=\"10\">P1</StartEntity>
  <EndEntity Type=\"P\" OriginalId=\"20\">P2</EndEntity>
  <ShowGraphicsType>All</ShowGraphicsType>
  <Line1Color>255,128,0,255</Line1Color>
  <Line2Color>0,0,255,255</Line2Color>
  <AzimColor>255,128,0,255</AzimColor>
  <ElevColor>255,128,0,255</ElevColor>
  <CompColor>255,128,0,255</CompColor>
  <DisplayText>Full</DisplayText>
  <TextLocation>Pairing Line</TextLocation>
  <TextFont>arialbd.ttf</TextFont>
  <TextPointSize>24</TextPointSize>
  <TextColor>255,255,255,255</TextColor>
  <TextXOffset>0</TextXOffset>
  <TextYOffset>0</TextYOffset>
  <CalculationGroup>
  <Name>Distances</Name>
  <CalculationType>Ground Rng</CalculationType>
  </CalculationGroup>
  </EntityPair>
  </RangeToolConfigurationFile>
"""

f = piSimdis.loadRangeConfiguration(configuration, True, "Example.xml")
if not f.isOk(10000):
	raise Exception("loadRangeConfiguration failed: " + f.responseDescription())

#check metadata
f = piSimdis.getFileMetadata(PISIMDISFile_pb2.FILE_FORMAT_RANGE_TOOL_CONFIG)
if not f.isOk(10000):
    raise Exception("getFileMetadata failed: " + f.responseDescription())
v = f.value();
if v[0].value != "1":
    raise Exception("Count did not equal 1")
if v[1].value != "Example.xml":
    raise Exception("Filename did not match Example.xml")

f = piSimdis.getRangeConfiguration()
if not f.isOk(10000):
	raise Exception("getRangeConfiguration failed: " + f.responseDescription())
returnedConfiguration = f.value()

# do compare after removing white space via re
if re.sub('[\s+]', '', configuration) != re.sub('[\s+]', '', returnedConfiguration):
    raise Exception("The returned configuration did not match the sent configuration")

print('Check SIMDIS for a ground range of 11073.1')
print('Will auto clear in 10 seconds.')
time.sleep(10)

f = piSimdis.removeAll()
if not f.isOk(10000):
	raise Exception("removeAll failed: " + f.responseDescription())

client.close()
print("Test successful")
