import math
import RemoteApi_pb2
import remoteapi

"""
TestSetSettings.py

Unit test for Remote API setSettings and getSettings functions
"""

client = remoteapi.DiscoveryClient().createClient()
if client is None:
    raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

# PISIMDIS::DoesSettingExist tests ----------------------------
f = piSimdis.doesSettingExist("HUD/Logo/Right Logo Filename")
if not f.isOk(1000):
    raise Exception("Failed to check HUD/Logo/Right Logo Filename")
if not f.value():
    raise Exception("HUD/Logo/Right Logo Filename returning non-existent")

f = piSimdis.doesSettingExist("Console/MadeUp Setting")
if not f.isOk(1000):
    raise Exception("Failed to check MadeUp Setting")
if f.value():
    raise Exception("MadeUp Setting returning existent")

# PISIMDIS::*SettingValueStr tests ----------------------------
setLogoPath = "plotxy4848.png"
f = piSimdis.setSettingValueStr("HUD/Logo/Right Logo Filename", setLogoPath)
if not f.isOk(1000):
    raise Exception("Failed to set HUD/Logo/Right Logo Filename")

f = piSimdis.getSettingValueStr("HUD/Logo/Right Logo Filename", "default")
if not f.isOk(1000):
    raise Exception("Failed to get HUD/Logo/Right Logo Filename")
if f.value() != setLogoPath:
    raise Exception("Fetched HUD/Logo/Right Logo Filename")

# PISIMDIS::*SettingValueInt tests ----------------------------
setBufferLines = 1001
f = piSimdis.setSettingValueInt("Console/Buffer Lines", setBufferLines)
if not f.isOk(1000):
    raise Exception("Failed to set Console/Buffer Lines")

f = piSimdis.getSettingValueInt("Console/Buffer Lines", 0)
if not f.isOk(1000):
    raise Exception("Failed to get Console/Buffer Lines")
if f.value() != setBufferLines:
    raise Exception("Fetched Console/Buffer Lines does not match")

# PISIMDIS::*SettingValueDouble tests ----------------------------
setLineWidth = 2.22
f = piSimdis.setSettingValueDouble("GOG/OGR Line Width", setLineWidth)
if not f.isOk(1000):
    raise Exception("Failed to set GOG/OGR Line Width")

f = piSimdis.getSettingValueDouble("GOG/OGR Line Width", 0.0)
if not f.isOk(1000):
    raise Exception("Failed to get GOG/OGR Line Width")
if f.value() != setLineWidth:
    raise Exception("Fetched GOG/OGR Line Width does not match")

# PISIMDIS::*SettingValueBool tests ----------------------------
setOnTop = True
f = piSimdis.setSettingValueBool("Console/Newest On Top", setOnTop)
if not f.isOk(1000):
    raise Exception("Failed to set Console/Newest On Top")

f = piSimdis.getSettingValueBool("Console/Newest On Top", False)
if not f.isOk(1000):
    raise Exception("Failed to get Console/Newest On Top")
if f.value() != setOnTop:
    raise Exception("Fetched Console/Newest On Top does not match")

# PISIMDIS::*SettingValueColor tests ----------------------------
setGOGColor1 = 0xffaaaaff
f = piSimdis.setSettingValueColor("GOG/Color 1", setGOGColor1)
if not f.isOk(1000):
    raise Exception("Failed to set GOG/Color 1")

f = piSimdis.getSettingValueColor("GOG/Color 1", 0x00000000)
if not f.isOk(1000):
    raise Exception("Failed to get GOG/Color 1")
if f.value() != setGOGColor1:
    raise Exception("Fetched GOG/Color 1 does not match")

# All done, successful if we got here
client.close()
print("Test successful")
