from collections import namedtuple
import RemoteApi_pb2
try:
  from . import PluginApi
except ImportError:
  import PluginApi

ReturnCodeOnly = lambda r: r.response.returnCode

class PIDataSender(object):
	def __init__(self, client):
		self.client_ = client

	def loadDataFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.loadDataFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def isScenarioInitialized(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.isScenarioInitialized = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.isScenarioInitialized)

	def deleteScenario(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.deleteScenario = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def initializeScenario(self, filenameOrServer, portNum, protocol, liveMode):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.initialize.filenameOrServer = filenameOrServer
		msg.piData.scenario.initialize.portNum = portNum
		msg.piData.scenario.initialize.protocol = protocol
		msg.piData.scenario.initialize.liveMode = liveMode
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setOriginLLA(self, latRad, lonRad, altMeters):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setOrigin.latRad = latRad
		msg.piData.scenario.setOrigin.lonRad = lonRad
		msg.piData.scenario.setOrigin.altMeters = altMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getOriginLLA(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getOriginLLA = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: [r.piData.scenario.originLLA.latRad, r.piData.scenario.originLLA.lonRad, r.piData.scenario.originLLA.altMeters] )

	def setReferenceYear(self, year):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setReferenceYear = year
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getReferenceYear(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getReferenceYear = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.referenceYear)

	def setDescription(self, desc):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setDescription = desc
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getDescription(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getDescription = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.text)

	def setScenarioFilename(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setScenarioFileName = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getScenarioFilename(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getScenarioFileName = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.text)

	def setClassification(self, classText, color):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setClassification.text = classText
		msg.piData.scenario.setClassification.color = color
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.text)

	def getClassification(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getClassification = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: [r.piData.scenario.classification.text, r.piData.scenario.classification.color])

	def setWind(self, speed, direction):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setWind.speed = speed
		msg.piData.scenario.setWind.direction = direction
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.text)

	def getWind(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getWind = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: [r.piData.scenario.wind.speed, r.piData.scenario.wind.direction] )

	def findPlatform(self, platformName):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findPlatform = platformName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.uniqueId)

	def findBeam(self, beamName):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findBeam = beamName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.uniqueId)

	def findGate(self, gateName):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findGate = gateName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.uniqueId)

	def findEntity(self, entityName):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findEntity = entityName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.uniqueId)

	def findObjectsByOriginalID(self, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findObjectsByOriginalID = originalId
		# Note that although the ID list is iterable, it is not a list; we convert to a list
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: map(lambda v : v, r.piData.scenario.getList.uniqueId))

	def clockAutoConfigureBeginEnd(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockAutoConfigureBeginEnd = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clockSetTime(self, timeValue, threshold=0.0):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setTime.time = timeValue
		msg.piData.scenario.setTime.threshold = threshold
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clockSetMode(self, mode, startTime=-1.0):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setClockMode.newMode = mode
		msg.piData.scenario.setClockMode.startTime = startTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clockGetMode(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getClockMode = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.clockMode)

	def clockGetBegin(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockGetBegin = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.time)

	def clockGetEnd(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockGetEnd = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.time)

	def clockIsPlaying(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockIsPlaying = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.state)

	def clockGetStep(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockGetStep = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.doubleValue)

	def clockSetStep(self, step):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockSetStep = step
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clockStop(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockStop = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clockPlay(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.clockPlay = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def convertTime(self, refYear, seconds, toYear):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.convertTime.refYear = refYear
		msg.piData.scenario.convertTime.seconds = seconds
		msg.piData.scenario.convertTime.toYear = toYear
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.time)

	def convertToScenarioTime(self, refYear, seconds):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.convertToScenarioTime.refYear = refYear
		msg.piData.scenario.convertToScenarioTime.seconds = seconds
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.time)

	def createProjector(self, host, projectorName, originalId, id = None):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.create.projectorName = projectorName
		msg.piData.projector.create.hostId = host
		msg.piData.projector.create.originalId = originalId
		if id is not None:
			msg.piData.projector.create.suppliedId = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.projector.projectorId)

	def removeEntity(self, host, force=False):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.removeEntity.id = host
		msg.piData.scenario.removeEntity.force = force
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addProjectorFov(self, host, time, fov):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.addFov.id = host
		msg.piData.projector.addFov.time = time
		msg.piData.projector.addFov.fov = fov
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setColor(self, host, color, time):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.addColor.id = host
		add = msg.piData.scenario.addColor.points.add()
		add.time = time
		add.color = color
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setStatus(self, host, time, state):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.addStatus.id = host
		add = msg.piData.scenario.addStatus.points.add()
		add.time = time
		add.status = state
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getStatus(self, host, startTime, endTime, includeInitial=False):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getStatus.id = host
		msg.piData.scenario.getStatus.startTime = startTime
		msg.piData.scenario.getStatus.endTime = endTime
		msg.piData.scenario.getStatus.includeInitial = includeInitial
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getStatus)

	def setProjectorRasterFilename(self, host, filename):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.setRasterFilename.projectorId = host
		msg.piData.projector.setRasterFilename.filename = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getProjectorRasterFilename(self, host):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.getRasterFilename = host
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.projector.filename)

	def setProjectorRasterID(self, projectorId, textureId):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.setProjectorRasterId.projectorId = projectorId
		msg.piData.projector.setProjectorRasterId.textureId = textureId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getProjectorHeader(self, host):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.getHeader = host
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r:
				namedtuple('Header', 'name originalId')(
					name = r.piData.projector.getHeader.name,
					originalId = r.piData.projector.getHeader.originalId
					))

	def getAllProjectorData(self, host):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.getData.projectorId = host
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.projector.points)

	def getProjectorData(self, host, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.projector.getData.projectorId = host
		msg.piData.projector.getData.startTime = startTime
		msg.piData.projector.getData.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.projector.points)

	def setObjectOffset(self, objectId, xyz):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setObjectOffsets.id = objectId
		msg.piData.scenario.setObjectOffsets.offsets.position.x = xyz[0]
		msg.piData.scenario.setObjectOffsets.offsets.position.y = xyz[1]
		msg.piData.scenario.setObjectOffsets.offsets.position.z = xyz[2]
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setObjectOrientationOffset(self, objectId, ori):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setObjectOffsets.id = objectId
		msg.piData.scenario.setObjectOffsets.offsets.orientation.x = ori[0]
		msg.piData.scenario.setObjectOffsets.offsets.orientation.y = ori[1]
		msg.piData.scenario.setObjectOffsets.offsets.orientation.z = ori[2]
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getObjectOffset(self, objectId):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getObjectOffsets = objectId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: [r.piData.scenario.objectOffsets.position.x, r.piData.scenario.objectOffsets.position.y, r.piData.scenario.objectOffsets.position.z] )

	def getObjectOrientationOffset(self, objectId):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getObjectOffsets = objectId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: [r.piData.scenario.objectOffsets.orientation.x, r.piData.scenario.objectOffsets.orientation.y, r.piData.scenario.objectOffsets.orientation.z] )

	def getObjectType(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getObjectType = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.type)

	def getObjectHost(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getObjectHost = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.uniqueId)

	def flushEntityData(self, id, type):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.flushEntityData.id = id
		msg.piData.scenario.flushEntityData.type = type
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getObjectList(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getObjectList = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: map(lambda v : v, r.piData.scenario.getList.uniqueId))

	def createDataTable(self, hostId, name):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.create.hostId = hostId
		msg.piData.table.create.name = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.table.uniqueId)

	def addColumn(self, tableId, name, type, unit):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.addColumns.id = tableId
		column = msg.piData.table.addColumns.columns.add()
		column.name = name
		column.type = type
		column.unit = unit
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.table.columnId)

	def addRow(self, tableId, time, values):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.addRows.id = tableId
		row = msg.piData.table.addRows.rows.add()
		row.time = time
		index = 0
		for value in values:
			cell = row.cells.add()
			cell.index = index
			index = index + 1
			cell.doubleValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getColumns(self, tableId):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.getColumns = tableId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.table.columns)

	def getRows(self, tableId, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.getRows.id = tableId
		msg.piData.table.getRows.startTime = startTime
		msg.piData.table.getRows.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.table.rows)

	def getTableName(self, tableId):
		msg = RemoteApi_pb2.Command()
		msg.piData.table.getTableInfo = tableId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.table.tableInfo.name)

	def getTableList(self, hostId):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getTableList = hostId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getList)

	def removeDataTable(self, tableId):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.removeDataTable = tableId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createPlatform(self, name, iconName, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.create.header.name = name
		msg.piData.platform.create.header.iconName = iconName
		msg.piData.platform.create.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.platform.uniqueId)

	def updatePlatformHeader(self, id, name, iconName, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.updateHeader.id = id
		msg.piData.platform.updateHeader.header.name = name
		msg.piData.platform.updateHeader.header.iconName = iconName
		msg.piData.platform.updateHeader.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getPlatformHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.platform.getHeader)

	def setPlatformRCSPatternFileName(self, id, filename):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.setPlatformRCSPatternFileName.id = id
		msg.piData.platform.setPlatformRCSPatternFileName.filename = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getPlatformRCSPatternFileName(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.getPlatformRCSPatternFileName = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.platform.text)

	def addPlatformData(self, id, time, lat, lon, alt, ori = None):
		msg = RemoteApi_pb2.Command()
		msg.piData.platform.addData.id = id
		msg.piData.platform.addData.frame.system = 3 # LLA
		point = msg.piData.platform.addData.points.add();
		point.time = time
		point.position.x = lat
		point.position.y = lon
		point.position.z = alt
		if ori is not None:
			point.orientation.psi_yaw = ori[0]
			point.orientation.theta_pitch = ori[1]
			point.orientation.phi_roll = ori[2]
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createBeam(self, hostId, name, type, hbw, vbw, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.beam.create.hostPlatform = hostId
		msg.piData.beam.create.header.name = name
		msg.piData.beam.create.header.type = type
		msg.piData.beam.create.header.horizontalBeamWidth = hbw
		msg.piData.beam.create.header.verticalBeamWidth = vbw
		msg.piData.beam.create.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.beam.uniqueId)

	def updateBeamHeader(self, id, name, type, hbw, vbw, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.beam.updateHeader.id = id
		msg.piData.beam.updateHeader.header.name = name
		msg.piData.beam.updateHeader.header.type = type
		msg.piData.beam.updateHeader.header.horizontalBeamWidth = hbw
		msg.piData.beam.updateHeader.header.verticalBeamWidth = vbw
		msg.piData.beam.updateHeader.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getBeamHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.beam.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.beam.getHeader)

	def setTarget(self, hostId, targetId, time):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setTargets.id = hostId
		target = msg.piData.scenario.setTargets.targets.add()
		target.targetId = targetId
		target.time = time;
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getTargets(self, hostId, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getTargets.id = hostId
		msg.piData.scenario.getTargets.startTime = startTime
		msg.piData.scenario.getTargets.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.target)

	def addBeamData(self, hostId, time, azimuth, elevation, length):
		msg = RemoteApi_pb2.Command()
		msg.piData.beam.addData.id = hostId
		pt = msg.piData.beam.addData.points.add()
		pt.time = time
		pt.azimuth = azimuth
		pt.elevation = elevation
		pt.length = length
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createGate(self, hostId, name, type, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.gate.create.hostBeam = hostId
		msg.piData.gate.create.header.name = name
		msg.piData.gate.create.header.type = type
		msg.piData.gate.create.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.gate.uniqueId)

	def updateGateHeader(self, id, name, type, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.gate.updateHeader.id = id
		msg.piData.gate.updateHeader.header.name = name
		msg.piData.gate.updateHeader.header.type = type
		msg.piData.gate.updateHeader.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getGateHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.gate.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.gate.getHeader)

	def createLob(self, hostId, name, type, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.lob.create.hostPlatform = hostId
		msg.piData.lob.create.header.name = name
		msg.piData.lob.create.header.type = type
		msg.piData.lob.create.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.lob.uniqueId)

	def updateLobHeader(self, id, name, type, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.lob.updateHeader.id = id
		msg.piData.lob.updateHeader.header.name = name
		msg.piData.lob.updateHeader.header.type = type
		msg.piData.lob.updateHeader.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getLobHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.lob.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.lob.getHeader)

	def getLobDrawStyle(self, id, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.lob.getStyle.id = id
		msg.piData.lob.getStyle.startTime = startTime
		msg.piData.lob.getStyle.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.lob.style)

	def addLobDrawStyle(self, id, time, color1, color2, stipple1, stipple2, pixelWidth):
		msg = RemoteApi_pb2.Command()
		msg.piData.lob.addDrawStyle.id = id
		style = msg.piData.lob.addDrawStyle.styles.add()
		style.time = time
		style.color1 = color1
		style.color2 = color2
		style.stipple1 = stipple1
		style.stipple2 = stipple2
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setHistoryLengths(self, id, points, seconds):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.setHistoryLengths.id = id
		msg.piData.scenario.setHistoryLengths.lengths.points = points
		msg.piData.scenario.setHistoryLengths.lengths.seconds = seconds
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getHistoryLengths(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getHistoryLengths = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.historyLengths)

	def getGenericDataList(self):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getGenericDataList = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: map(lambda v : v, r.piData.scenario.getList.uniqueId))

	def addGenericData(self, id, time, tag, data):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.addGeneric.hostObjectOrScenario = id
		point = msg.piData.scenario.addGeneric.points.add()
		point.time = time
		point.tag = tag
		point.data = data
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def deleteGenericData(self, id, tag):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.deleteGenericData.id = id
		point = msg.piData.scenario.deleteGenericData.tag = tag
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def retrieveCurrentCategoryData(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.retrieveCurrentCategoryData = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getCategoryData)

	def addCategoryData(self, id, time, category, value):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.addCategory.id = id
		point = msg.piData.scenario.addCategory.points.add()
		point.time = time
		point.category = category
		point.value = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createCustomRendering(self, hostId, name, renderer, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.customRendering.create.host = hostId
		msg.piData.customRendering.create.header.name = name
		msg.piData.customRendering.create.header.renderer = renderer
		msg.piData.customRendering.create.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.customRendering.uniqueId)

	def updateCustomRenderingHeader(self, id, name, renderer, originalId):
		msg = RemoteApi_pb2.Command()
		msg.piData.customRendering.updateHeader.id = id
		msg.piData.customRendering.updateHeader.header.name = name
		msg.piData.customRendering.updateHeader.header.renderer = renderer
		msg.piData.customRendering.updateHeader.header.originalId = originalId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getCustomRenderingHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.customRendering.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.customRendering.getHeader)

	def getEntitiesByCategoryString(self, categoryfilterstring):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getEntitiesByCategoryString = categoryfilterstring
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getList.uniqueId)

	def getEntitiesByCategoryAndValue(self, category, value, types):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getEntitiesByCategoryAndValue.category = category
		msg.piData.scenario.getEntitiesByCategoryAndValue.value = value
		msg.piData.scenario.getEntitiesByCategoryAndValue.types = types
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getList.uniqueId)

	def createLaser(self, hostId, name, originalId, type, coordSys, originCoord, status):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.create.hostPlatform = hostId
		msg.piData.laser.create.header.name = name
		msg.piData.laser.create.header.originalId = originalId
		msg.piData.laser.create.header.type = type
		msg.piData.laser.create.header.coordSystem = coordSys
		msg.piData.laser.create.header.originLlaX = originCoord[0]
		msg.piData.laser.create.header.originLlaY = originCoord[1]
		msg.piData.laser.create.header.originLlaZ = originCoord[2]
		msg.piData.laser.create.status = status
		msg.piData.laser.create.suppliedId = originalId # TODO are these the same thing?
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.laser.uniqueId)

	def addLaserData(self, id, time, azimuth, elevation, roll):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.addData.id = id
		point = msg.piData.laser.addData.points.add()
		point.time = time
		point.azimuth = azimuth
		point.elevation = elevation
		point.roll = roll
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addLaserStyle(self, id, time, maxRange, xOffset, yOffset, zOffset, width):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.addStyle.id = id
		style = msg.piData.laser.addStyle.styles.add()
		style.time = time
		style.maxRange = maxRange
		style.xOffset = xOffset
		style.yOffset = yOffset
		style.zOffset = zOffset
		style.width = width
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getLaserData(self, id, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.getData.id = id
		msg.piData.laser.getData.startTime = startTime
		msg.piData.laser.getData.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.laser.points)

	def getLaserHeader(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.getHeader = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.laser.getHeader)

	def getLaserStyles(self, id, startTime, endTime):
		msg = RemoteApi_pb2.Command()
		msg.piData.laser.getStyles.id = id
		msg.piData.laser.getStyles.startTime = startTime
		msg.piData.laser.getStyles.endTime = endTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.laser.styles)

	def getCurrentGenericData(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.getCurrentGenericData = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getGenericData)

	def findObject(self, objType, name):
		msg = RemoteApi_pb2.Command()
		msg.piData.scenario.findObject.objType = objType
		msg.piData.scenario.findObject.name = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piData.scenario.getList.uniqueId)
